/**
 * Copyright: Copyright (c) 2009 Jacob Carlborg.
 * Authors: Jacob Carlborg
 * Version: Initial created: Sep 24, 2009 
 * License: $(LINK2 http://www.boost.org/LICENSE_1_0.txt, Boost Software License 1.0)
 */
module dstep.appkit.NSDatePickerCell;

import dstep.appkit.NSActionCell;
import dstep.appkit.NSColor;
import dstep.foundation.NSCalendar;
import dstep.foundation.NSDate;
import dstep.foundation.NSLocale;
import dstep.foundation.NSObjCRuntime;
import dstep.foundation.NSTimeZone;
import dstep.objc.bridge.Bridge;
import dstep.objc.objc;

alias NSUInteger NSDatePickerStyle;
alias NSUInteger NSDatePickerMode;
alias NSUInteger NSDatePickerElementFlags;

enum
{
	NSTextFieldAndStepperDatePickerStyle = 0,
	NSClockAndCalendarDatePickerStyle = 1,
	NSTextFieldDatePickerStyle = 2
}

enum
{
	NSSingleDateMode = 0,
	NSRangeDateMode = 1
}

enum
{
	NSHourMinuteDatePickerElementFlag = 0x000c,
	NSHourMinuteSecondDatePickerElementFlag = 0x000e,
	NSTimeZoneDatePickerElementFlag = 0x0010,
	NSYearMonthDatePickerElementFlag = 0x00c0,
	NSYearMonthDayDatePickerElementFlag = 0x00e0,
	NSEraDatePickerElementFlag = 0x0100
}

class NSDatePickerCell : NSActionCell
{
	mixin (ObjcWrap);

	uint datePickerStyle ()
	{
		return invokeObjcSelf!(uint, "datePickerStyle");
	}

	void setDatePickerStyle (uint newStyle)
	{
		return invokeObjcSelf!(void, "setDatePickerStyle:", uint)(newStyle);
	}

	bool drawsBackground ()
	{
		return invokeObjcSelf!(bool, "drawsBackground");
	}

	void setDrawsBackground (bool flag)
	{
		return invokeObjcSelf!(void, "setDrawsBackground:", bool)(flag);
	}

	NSColor backgroundColor ()
	{
		return invokeObjcSelf!(NSColor, "backgroundColor");
	}

	void setBackgroundColor (NSColor color)
	{
		return invokeObjcSelf!(void, "setBackgroundColor:", NSColor)(color);
	}

	NSColor textColor ()
	{
		return invokeObjcSelf!(NSColor, "textColor");
	}

	void setTextColor (NSColor color)
	{
		return invokeObjcSelf!(void, "setTextColor:", NSColor)(color);
	}

	uint datePickerMode ()
	{
		return invokeObjcSelf!(uint, "datePickerMode");
	}

	void setDatePickerMode (uint newMode)
	{
		return invokeObjcSelf!(void, "setDatePickerMode:", uint)(newMode);
	}

	uint datePickerElements ()
	{
		return invokeObjcSelf!(uint, "datePickerElements");
	}

	void setDatePickerElements (uint elementFlags)
	{
		return invokeObjcSelf!(void, "setDatePickerElements:", uint)(elementFlags);
	}

	NSCalendar calendar ()
	{
		return invokeObjcSelf!(NSCalendar, "calendar");
	}

	void setCalendar (NSCalendar newCalendar)
	{
		return invokeObjcSelf!(void, "setCalendar:", NSCalendar)(newCalendar);
	}

	NSLocale locale ()
	{
		return invokeObjcSelf!(NSLocale, "locale");
	}

	void setLocale (NSLocale newLocale)
	{
		return invokeObjcSelf!(void, "setLocale:", NSLocale)(newLocale);
	}

	NSTimeZone timeZone ()
	{
		return invokeObjcSelf!(NSTimeZone, "timeZone");
	}

	void setTimeZone (NSTimeZone newTimeZone)
	{
		return invokeObjcSelf!(void, "setTimeZone:", NSTimeZone)(newTimeZone);
	}

	NSDate dateValue ()
	{
		return invokeObjcSelf!(NSDate, "dateValue");
	}

	void setDateValue (NSDate newStartDate)
	{
		return invokeObjcSelf!(void, "setDateValue:", NSDate)(newStartDate);
	}

	double timeInterval ()
	{
		return invokeObjcSelf!(double, "timeInterval");
	}

	void setTimeInterval (double newTimeInterval)
	{
		return invokeObjcSelf!(void, "setTimeInterval:", double)(newTimeInterval);
	}

	NSDate minDate ()
	{
		return invokeObjcSelf!(NSDate, "minDate");
	}

	void setMinDate (NSDate date)
	{
		return invokeObjcSelf!(void, "setMinDate:", NSDate)(date);
	}

	NSDate maxDate ()
	{
		return invokeObjcSelf!(NSDate, "maxDate");
	}

	void setMaxDate (NSDate date)
	{
		return invokeObjcSelf!(void, "setMaxDate:", NSDate)(date);
	}

	Object delegate_ ()
	{
		return invokeObjcSelf!(Object, "delegate");
	}

	void setDelegate (Object anObject)
	{
		return invokeObjcSelf!(void, "setDelegate:", Object)(anObject);
	}
}

const TNSDatePickerCellDelegate = `

	void datePickerCell (NSDatePickerCell aDatePickerCell, NSDate** proposedDateValue, NSTimeInterval* proposedTimeInterval)
	{
		return invokeObjcSelf!(void, "datePickerCell:validateProposedDateValue:timeInterval:", NSDatePickerCell, NSDate**, NSTimeInterval*)(aDatePickerCell, proposedDateValue, proposedTimeInterval);
	}

	//mixin ObjcBindMethod!(datePickerCell, "datePickerCell:validateProposedDateValue:timeInterval:");

`;

