/**
 * Copyright: Copyright (c) 2009 Jacob Carlborg.
 * Authors: Jacob Carlborg
 * Version: Initial created: Sep 24, 2009 
 * License: $(LINK2 http://www.boost.org/LICENSE_1_0.txt, Boost Software License 1.0)
 */
module dstep.appkit.NSTextList;

import dstep.foundation.NSCoder;
import dstep.foundation.NSObject;
import dstep.foundation.NSString;
import dstep.foundation.NSObjCRuntime;
import dstep.foundation.NSZone;
import dstep.objc.bridge.Bridge;
import dstep.objc.objc;

enum
{
	NSTextListPrependEnclosingMarker = (1 << 0)
}

class NSTextList : NSObject, INSCoding, INSCopying
{
	mixin (ObjcWrap);
	
	this (NSCoder aDecoder)
	{
		super(typeof(this).alloc.initWithCoder(aDecoder).objcObject);
	}
	
	void encodeWithCoder (NSCoder aCoder)
	{
		return invokeObjcSelf!(void, "encodeWithCoder:", NSCoder)(aCoder);
	}
	
	typeof(this) initWithCoder (NSCoder aDecoder)
	{
		return invokeObjcSelf!(typeof(this), "initWithCoder:", NSCoder)(aDecoder);
	}
	
	typeof(this) copyWithZone (NSZone* zone)
	{
		return invokeObjcSelf!(typeof(this), "copyWithZone:", NSZone*)(zone);
	}

	NSTextList initWithMarkerFormat (NSString format, NSUInteger mask)
	{
		id result = invokeObjcSelf!(id, "initWithMarkerFormat:options:", NSString, NSUInteger)(format, mask);
		return result is this.objcObject ? this : (result !is null ? new NSTextList(result) : null);
	}

	this (NSString format, NSUInteger mask)
	{
		super(NSTextList.alloc.initWithMarkerFormat(format, mask).objcObject);
	}

	NSString markerFormat ()
	{
		return invokeObjcSelf!(NSString, "markerFormat");
	}

	NSUInteger listOptions ()
	{
		return invokeObjcSelf!(NSUInteger, "listOptions");
	}

	NSString markerForItemNumber (NSInteger itemNum)
	{
		return invokeObjcSelf!(NSString, "markerForItemNumber:", NSInteger)(itemNum);
	}

}

