/**
 * Copyright: Copyright (c) 2009 Jacob Carlborg.
 * Authors: Jacob Carlborg
 * Version: Initial created: Aug 3, 2009 
 * License: $(LINK2 http://www.boost.org/LICENSE_1_0.txt, Boost Software License 1.0)
 */
module dstep.foundation.NSSpellServer;

import dstep.foundation.NSArray;
import dstep.foundation.NSObject;
import dstep.foundation.NSRange;
import dstep.foundation.NSString;
import dstep.objc.bridge.Bridge;
import dstep.objc.objc;



import bindings = dstep.foundation.NSSpellServer_bindings;

private
{
	NSString NSGrammarRange_;
	NSString NSGrammarUserDescription_;
	NSString NSGrammarCorrections_;
}

NSString NSGrammarRange ()
{
	if (NSGrammarRange_)
		return NSGrammarRange_;
		
	return NSGrammarRange_ = new NSString(bindings.NSGrammarRange);
}

NSString NSGrammarUserDescription ()
{
	if (NSGrammarUserDescription_)
		return NSGrammarUserDescription_;
		
	return NSGrammarUserDescription_ = new NSString(bindings.NSGrammarUserDescription);
}

NSString NSGrammarCorrections ()
{
	if (NSGrammarCorrections_)
		return NSGrammarCorrections_;
		
	return NSGrammarCorrections_ = new NSString(bindings.NSGrammarCorrections);
}

const TNSSpellServerDelegate = `
	
	NSRange spellServer (NSSpellServer sender, NSString stringToCheck, NSString language, NSInteger* wordCount, bool countOnly)
	{
		return invokeObjcSelf!(NSRange, "spellServer:findMisspelledWordInString:language:wordCount:countOnly:", NSSpellServer, NSString, NSString, NSInteger*, bool)(sender, stringToCheck, language, wordCount, countOnly);
	}
	
	NSArray spellServer (NSSpellServer sender, NSString word, NSString language)
	{
		return invokeObjcSelf!(NSArray, "spellServer:suggestGuessesForWord:inLanguage:", NSSpellServer, NSString, NSString)(sender, word, language);
	}
	
	void spellServer_didLearnWord_inLanguage (NSSpellServer sender, NSString word, NSString language)
	{
		return invokeObjcSelf!(void, "spellServer:didLearnWord:inLanguage:", NSSpellServer, NSString, NSString)(sender, word, language);
	}
	
	void spellServer (NSSpellServer sender, NSString word, NSString language)
	{
		return invokeObjcSelf!(void, "spellServer:didForgetWord:inLanguage:", NSSpellServer, NSString, NSString)(sender, word, language);
	}
	
	NSArray spellServer (NSSpellServer sender, NSRange range, NSString string, NSString language)
	{
		return invokeObjcSelf!(NSArray, "spellServer:suggestCompletionsForPartialWordRange:inString:language:", NSSpellServer, NSRange, NSString, NSString)(sender, range, string, language);
	}
	
	NSRange spellServer (NSSpellServer sender, NSString stringToCheck, NSString language, out NSArray details)
	{
		id det = new objc_object;
		
		NSRange result = invokeObjcSelf!(NSRange, "spellServer:checkGrammarInString:language:details:", NSSpellServer, NSString, NSString, id*)(sender, stringToCheck, language, &det);
		
		if (det)
			details = new NSArray(det);
			
		return result;
	}
	
	//mixin ObjcBindMethod!(spellServer, "spellServer:findMisspelledWordInString:language:wordCount:countOnly:");
	//mixin ObjcBindMethod!(spellServer, "spellServer:suggestGuessesForWord:inLanguage:");
	//mixin ObjcBindMethod!(spellServer, "spellServer:didLearnWord:inLanguage:");
	//mixin ObjcBindMethod!(spellServer, "spellServer:didForgetWord:inLanguage:");
	//mixin ObjcBindMethod!(spellServer, "spellServer:suggestCompletionsForPartialWordRange:inString:language:");
	//mixin ObjcBindMethod!(spellServer, "spellServer:checkGrammarInString:language:details:");
`;

class NSSpellServer : NSObject
{
	mixin (ObjcWrap);
	
	this ()
	{
		super(typeof(this).alloc.init.objcObject);
	}
	
	typeof(this) init ()
	{
		return invokeObjcSelf!(typeof(this), "init");
	}

	void setDelegate (Object anObject)
	{
		return invokeObjcSelf!(void, "setDelegate:", Object)(anObject);
	}

	Object delegate_ ()
	{
		return invokeObjcSelf!(Object, "delegate");
	}

	bool registerLanguage (NSString language, NSString vendor)
	{
		return invokeObjcSelf!(bool, "registerLanguage:byVendor:", NSString, NSString)(language, vendor);
	}

	bool isWordInUserDictionaries (NSString word, bool flag)
	{
		return invokeObjcSelf!(bool, "isWordInUserDictionaries:caseSensitive:", NSString, bool)(word, flag);
	}

	void run ()
	{
		return invokeObjcSelf!(void, "run");
	}
}